#!/usr/bin/python3

#
#   Developer : Alexey Zakharov (alexey.zakharov@vectioneer.com)
#   All rights reserved. Copyright (c) 2017-2020 VECTIONEER.
#

import motorcortex
import time
import threading
import logging


def onLog(val):
    print(val[0].value)


def onVar(val):
    print(val[0].value)


def main(id):
    logging.basicConfig(level=logging.DEBUG)

    # Creating empty object for parameter tree
    parameter_tree = motorcortex.ParameterTree()

    # Loading protobuf types and hashes
    motorcortex_types = motorcortex.MessageTypes()

    # Open request connection
    req, sub = motorcortex.connect("wss://192.168.2.100:5568:5567", motorcortex_types, parameter_tree,
                                   certificate="mcx.cert.pem", timeout_ms=1000,
                                   login="", password="")

    start = time.time()
    for x in range(1000):
        # req.setParameter("root/Control/dummyBool", True)
        req.setParameter("root/Control/dummyDouble", x)
        r = req.getParameter("root/Control/dummyDouble")
        # print(r.get())
    end = time.time()

    print("total time: {}".format(end - start))

    # time.sleep(15)

    req.close()
    sub.close()


if __name__ == '__main__':
    # creating thread
    t1 = threading.Thread(target=main, args=(1,))
    t1.start()
    t1.join()



#!/usr/bin/python3

#
#   Developer : Alexey Zakharov (alexey.zakharov@vectioneer.com)
#   All rights reserved. Copyright (c) 2016 VECTIONEER.
#

from motorcortex.version import __version__
from motorcortex.parameter_tree import ParameterTree
from motorcortex.message_types import MessageTypes
from motorcortex.request import Request, ConnectionState
from motorcortex.reply import Reply
from motorcortex.subscribe import Subscribe
from motorcortex.subscription import Subscription
from motorcortex.setup_logger import logger


def parseUrl(url):
    end = url.rfind(':')
    start = url.rfind(':', 0, end)
    req_port = int(url[start + 1:end])
    sub_port = int(url[end + 1:])
    address = url[:start]
    return address, req_port, sub_port


def connect(url, motorcortex_types, param_tree, reconnect=True, **kwargs):
    if reconnect and not kwargs.get("state_update"):
        def stateUpdate(req, sub, state):
            if state == ConnectionState.CONNECTION_OK:
                req.login(kwargs.get("login"), kwargs.get("password")).get()
                sub.resubscribe()

        kwargs.update(state_update=stateUpdate)

    # Parse address
    address, req_port, sub_port = parseUrl(url)
    # Open request connection
    req = Request(motorcortex_types, param_tree)
    kwargs_copy = kwargs.copy()
    kwargs_copy.update(state_update=None)
    if not req.connect("{}:{}".format(address, req_port), **kwargs_copy).get():
        raise RuntimeError("Failed to establish request connection: {}:{}".format(address, req_port))
    # Open subscribe connection
    sub = Subscribe(req, motorcortex_types)
    if not sub.connect("{}:{}".format(address, sub_port), **kwargs).get():
        raise RuntimeError("Failed to establish subscribe connection: {}:{}".format(address, sub_port))
    # Login
    login_reply = req.login(kwargs['login'], kwargs['password'])
    login_reply_msg = login_reply.get()

    motorcortex_msg = motorcortex_types.motorcortex()
    if not login_reply_msg.status == motorcortex_msg.OK:
        raise RuntimeError("Login failed, status: {}".format(login_reply_msg.status))

    # Requesting a parameter tree
    param_tree_reply = req.getParameterTree()
    tree = param_tree_reply.get()
    param_tree.load(tree)

    return req, sub


def statusToStr(motorcortex_msg, code):
    """Converts status codes to a readable message.

        Args:
            motorcortex_msg(Module): refernce to a motorcortex module
            code(int): status code

        Returns:
            str: status message

        Examples:
            >>> login_reply = req.login("admin", "iddqd")
            >>> login_reply_msg = login_reply.get()
            >>> if login_reply_msg.status != motorcortex_msg.OK:
            >>>     print(motorcortex.statusToStr(motorcortex_msg, login_reply_msg.status))

    """

    status = 'Unknown code'
    if code == motorcortex_msg.OK:
        status = 'Success'
    elif code == motorcortex_msg.FAILED:
        status = 'Failed'
    elif code == motorcortex_msg.FAILED_TO_DECODE:
        status = 'Failed to decode request'
    elif code == motorcortex_msg.SUB_LIST_IS_FULL:
        status = 'Failed to subscribe, subscription list is full'
    elif code == motorcortex_msg.WRONG_PARAMETER_PATH:
        status = 'Failed to find parameter'
    elif code == motorcortex_msg.FAILED_TO_SET_REQUESTED_FRQ:
        status = 'Failed to set requested frequency'
    elif code == motorcortex_msg.FAILED_TO_OPEN_FILE:
        status = 'Failed to open file'
    elif code == motorcortex_msg.READ_ONLY_MODE:
        status = 'Logged in, read-only mode'
    elif code == motorcortex_msg.WRONG_PASSWORD:
        status = 'Wrong login or password'
    elif code == motorcortex_msg.USER_NOT_LOGGED_IN:
        status = 'Operation is not permitted, user is not logged in'
    elif code == motorcortex_msg.PERMISSION_DENIED:
        status = 'Operation is not permitted, user has no rights'

    status += str(' (%s)' % hex(code))

    return status



#!/usr/bin/python3

from pynng import Sub0, TLSConfig
import threading
import time


def test(identifier, url, crt):
    print("open connection id: {}".format(identifier))
    tls_config = TLSConfig(TLSConfig.MODE_CLIENT, ca_files=crt)
    socket = Sub0(recv_timeout=1000000, tls_config=tls_config)
    socket.dial(url)
    socket.subscribe('')

    for i in range(1000):
        socket.recv()

    socket.close()

    print("close connection id: {}".format(identifier))


if __name__ == '__main__':
    # creating thread

    threads = []
    for i in range(50):
        threads.append(threading.Thread(target=test, args=(i, 'wss://localhost:5555', 'mcx.cert.pem')))
        threads[-1].start()
        time.sleep(0.01)

    for t in threads:
        t.join()



#!/usr/bin/python3

#
#   Developer : Alexey Zakharov (alexey.zakharov@vectioneer.com)
#   All rights reserved. Copyright (c) 2016 VECTIONEER.
#

from motorcortex.request import Request, Reply, ConnectionState
from motorcortex.subscription import Subscription
from pynng import Sub0, TLSConfig
from concurrent.futures import ThreadPoolExecutor
from queue import Queue


class Subscribe:
    """Subscribe class is used to receive continuous parameter updates from motorcortex server.

        Subscribe class simplifies creating and removing subscription groups.

        Args:
            req(Request): reference to a Request instance
            protobuf_types(MessageTypes): reference to a MessageTypes instance

    """

    def __init__(self, req, protobuf_types):
        self.__socket = None
        self.__connected_lock = None
        self.__url = None
        self.__req = req
        self.__protobuf_types = protobuf_types
        self.__subscriptions = dict()
        self.__pool = ThreadPoolExecutor()
        self.__clb = ThreadPoolExecutor(max_workers=1)
        self.__connection_state = ConnectionState.DISCONNECTED

    def connect(self, url, **kwargs):
        """Open a subscribe connection.

            Args:
                url(str): motorcortex server URL

            Returns:
                bool: True - if connected, False othesudo apt-get install python-devrwise
        """

        self.__connection_state = ConnectionState.CONNECTING
        conn_timeout_ms, recv_timeout_ms, certificate, state_update = Request.parse(**kwargs)
        if recv_timeout_ms == 0:
            recv_timeout_ms = 500

        self.__url = url
        tls_config = None
        if certificate:
            tls_config = TLSConfig(TLSConfig.MODE_CLIENT, ca_files=certificate)

        self.__socket = Sub0(recv_timeout=recv_timeout_ms, tls_config=tls_config)

        self.__connected_lock = Queue()

        def pre_connect_cb(pipe):
            self.__connected_lock.put(True)
            self.__connection_state = ConnectionState.CONNECTION_OK
            if state_update:
                self.__clb.submit(state_update, self.__req, self, self.connectionState())

        def post_remove_cb(pipe):
            if self.__connection_state == ConnectionState.DISCONNECTING:
                self.__connection_state = ConnectionState.DISCONNECTED
            elif self.__connection_state == ConnectionState.CONNECTING:
                self.__connection_state = ConnectionState.CONNECTION_FAILED
            elif self.__connection_state == ConnectionState.CONNECTION_OK:
                self.__connection_state = ConnectionState.CONNECTION_LOST
            self.__connected_lock.put(False)
            if state_update:
                self.__clb.submit(state_update, self.__req, self, self.connectionState())

        self.__socket.add_pre_pipe_connect_cb(pre_connect_cb)
        self.__socket.add_post_pipe_remove_cb(post_remove_cb)
        self.__socket.dial(url)

        self.__pool.submit(self.run, self.__socket)

        return Reply(self.__pool.submit(Request.waitForConnection, self.__connected_lock,
                                        conn_timeout_ms / 1000.0))

    def close(self):
        """Close connection to the server"""
        self.__connection_state = ConnectionState.DISCONNECTING
        if self.__connected_lock:
            self.__connected_lock.put(False)
        self.__socket.close()
        self.__pool.shutdown(wait=True)

    def run(self, socket):
        while self.__socket:
            buffer = socket.recv()
            if buffer:
                sub_id_buf = buffer[:4]
                protocol_version = sub_id_buf[3]
                sub_id = sub_id_buf[0] + (sub_id_buf[1] << 8) + (sub_id_buf[2] << 16)
                sub = self.__subscriptions.get(sub_id)
                if sub:
                    length = len(buffer)
                    if protocol_version == 1:
                        sub._updateProtocol1(buffer[4:], length - 4)
                    elif protocol_version == 0:
                        sub._updateProtocol0(buffer[4:], length - 4)
                    else:
                        print('Unknown protocol type: %d', protocol_version)

        print('Subscribe connection closed')

    def subscribe(self, param_list, group_alias, frq_divider=1):
        """Create a subscription group for a list of the parameters.

            Args:
                param_list(list(str)): list of the parameters to subscribe to
                group_alias(str): name of the group
                frq_divider(int): frequency divider is a downscaling factor for the group publish rate

            Returns:
                  Subscription: A subscription handle, which acts as a JavaScript Promise,
                  it is resolved when subscription is ready or failed. After the subscription
                  is ready the handle is used to retrieve latest data.
        """

        subscription = Subscription(group_alias, self.__protobuf_types, frq_divider)
        reply = self.__req.createGroup(param_list, group_alias, frq_divider)
        reply.then(self.__complete, subscription, self.__socket).catch(
            subscription._failed)

        return subscription

    def unsubscribe(self, subscription):
        """Unsubscribe from the group.

            Args:
                subscription(Subscription): subscription handle

            Returns:
                  Reply: Returns a Promise, which resolves when the unsubscribe
                  operation is complete, fails otherwise.

        """
        sub_id = subscription.id()
        sub_id_buf = Subscribe.__idBuf(subscription.id())

        # stop receiving sub
        try:
            self.__socket.unsubscribe(sub_id_buf)
        except:
            pass

        # find and remove subscription
        if sub_id in self.__subscriptions:
            sub = self.__subscriptions[sub_id]
            # stop sub update thread
            sub.done()
            del self.__subscriptions[sub_id]

        # send remove group request to the server
        return self.__req.removeGroup(subscription.alias())

    def connectionState(self):
        return self.__connection_state

    def resubscribe(self):
        old_sub = self.__subscriptions.copy()
        self.__subscriptions.clear()
        for i in old_sub:
            s = old_sub[i]
            try:
                # unsubscribe from the old group
                self.__socket.unsubscribe(Subscribe.__idBuf(s.id()))
            except:
                pass
            # subscribe again, update id
            msg = self.__req.createGroup(s.layout(), s.alias(), s.frqDivider()).get()
            s._updateId(msg.id)
            self.__socket.subscribe(Subscribe.__idBuf(s.id()))
            self.__subscriptions[s.id()] = s

    @staticmethod
    def __idBuf(msg_id):
        return bytes([msg_id & 0xff, (msg_id >> 8) & 0xff, (msg_id >> 16) & 0xff])

    def __complete(self, msg, subscription, socket):
        if subscription._complete(msg):
            id_buf = Subscribe.__idBuf(msg.id)
            socket.subscribe(id_buf)
            self.__subscriptions[msg.id] = subscription



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: motorcortex.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='motorcortex.proto',
  package='motorcortex',
  syntax='proto2',
  serialized_options=None,
  serialized_pb=_b('\n\x11motorcortex.proto\x12\x0bmotorcortex\"i\n\x0fParameterOffset\x12\x36\n\x04type\x18\x01 \x02(\x0e\x32\x17.motorcortex.OffsetType:\x0fOFFSET_ELEMENTS\x12\x0e\n\x06offset\x18\x02 \x02(\r\x12\x0e\n\x06length\x18\x03 \x02(\r\"f\n\x05\x45rror\x12\x11\n\ttimestamp\x18\x01 \x02(\x06\x12\x14\n\x0c\x65rror_number\x18\x02 \x02(\x07\x12\x13\n\x0b\x65rror_level\x18\x03 \x02(\x07\x12\x11\n\tsubsystem\x18\x04 \x02(\x07\x12\x0c\n\x04info\x18\x05 \x02(\x07\"a\n\tErrorList\x12\"\n\x06\x65rrors\x18\x01 \x03(\x0b\x32\x12.motorcortex.Error\x12\x18\n\x10number_of_errors\x18\x02 \x02(\x07\x12\x16\n\x0eupdate_counter\x18\x03 \x02(\x07\"\x8a\x02\n\rParameterInfo\x12\n\n\x02id\x18\x01 \x02(\r\x12\x11\n\tdata_type\x18\x02 \x02(\r\x12\x11\n\tdata_size\x18\x03 \x02(\r\x12\x1a\n\x12number_of_elements\x18\x04 \x02(\r\x12\r\n\x05\x66lags\x18\x05 \x02(\r\x12\x13\n\x0bpermissions\x18\x06 \x02(\r\x12.\n\nparam_type\x18\x07 \x02(\x0e\x32\x1a.motorcortex.ParameterType\x12(\n\x08group_id\x18\x08 \x02(\x0e\x32\x16.motorcortex.UserGroup\x12\x1f\n\x04unit\x18\t \x02(\x0e\x32\x11.motorcortex.Unit\x12\x0c\n\x04path\x18\n \x02(\t\"\x94\x01\n\x12GroupParameterInfo\x12\r\n\x05index\x18\x01 \x02(\r\x12\x0e\n\x06offset\x18\x02 \x02(\r\x12\x0c\n\x04size\x18\x03 \x02(\r\x12(\n\x04info\x18\x04 \x02(\x0b\x32\x1a.motorcortex.ParameterInfo\x12\'\n\x06status\x18\x05 \x02(\x0e\x32\x17.motorcortex.StatusCode\"1\n\x06Header\x12\x14\n\x0c\x66rameCounter\x18\x01 \x02(\x07\x12\x11\n\ttimestamp\x18\x02 \x02(\x06\"Z\n\x08GroupMsg\x12#\n\x06header\x18\x01 \x01(\x0b\x32\x13.motorcortex.Header\x12)\n\x06params\x18\x02 \x03(\x0b\x32\x19.motorcortex.ParameterMsg\"Y\n\tStatusMsg\x12#\n\x06header\x18\x01 \x01(\x0b\x32\x13.motorcortex.Header\x12\'\n\x06status\x18\x02 \x02(\x0e\x32\x17.motorcortex.StatusCode\"P\n\x08LoginMsg\x12#\n\x06header\x18\x01 \x01(\x0b\x32\x13.motorcortex.Header\x12\r\n\x05login\x18\x02 \x02(\t\x12\x10\n\x08password\x18\x03 \x02(\t\"9\n\x12GetSessionTokenMsg\x12#\n\x06header\x18\x01 \x01(\x0b\x32\x13.motorcortex.Header\"n\n\x0fSessionTokenMsg\x12#\n\x06header\x18\x01 \x01(\x0b\x32\x13.motorcortex.Header\x12\r\n\x05token\x18\x02 \x02(\t\x12\'\n\x06status\x18\x03 \x02(\x0e\x32\x17.motorcortex.StatusCode\"G\n\x11RestoreSessionMsg\x12#\n\x06header\x18\x01 \x01(\x0b\x32\x13.motorcortex.Header\x12\r\n\x05token\x18\x02 \x02(\t\"0\n\tLogoutMsg\x12#\n\x06header\x18\x01 \x01(\x0b\x32\x13.motorcortex.Header\":\n\x13GetParameterTreeMsg\x12#\n\x06header\x18\x01 \x01(\x0b\x32\x13.motorcortex.Header\"\x9a\x01\n\x10ParameterTreeMsg\x12#\n\x06header\x18\x01 \x01(\x0b\x32\x13.motorcortex.Header\x12*\n\x06params\x18\x02 \x03(\x0b\x32\x1a.motorcortex.ParameterInfo\x12\x0c\n\x04hash\x18\x03 \x02(\r\x12\'\n\x06status\x18\x04 \x02(\x0e\x32\x17.motorcortex.StatusCode\">\n\x17GetParameterTreeHashMsg\x12#\n\x06header\x18\x01 \x01(\x0b\x32\x13.motorcortex.Header\"r\n\x14ParameterTreeHashMsg\x12#\n\x06header\x18\x01 \x01(\x0b\x32\x13.motorcortex.Header\x12\x0c\n\x04hash\x18\x02 \x02(\r\x12\'\n\x06status\x18\x03 \x02(\x0e\x32\x17.motorcortex.StatusCode\"h\n\x0e\x43reateGroupMsg\x12#\n\x06header\x18\x01 \x01(\x0b\x32\x13.motorcortex.Header\x12\x13\n\x0b\x66rq_divider\x18\x02 \x02(\r\x12\r\n\x05\x61lias\x18\x03 \x02(\t\x12\r\n\x05paths\x18\x04 \x03(\t\"\xaa\x01\n\x0eGroupStatusMsg\x12#\n\x06header\x18\x01 \x01(\x0b\x32\x13.motorcortex.Header\x12\n\n\x02id\x18\x02 \x02(\r\x12\r\n\x05\x61lias\x18\x03 \x02(\t\x12/\n\x06params\x18\x04 \x03(\x0b\x32\x1f.motorcortex.GroupParameterInfo\x12\'\n\x06status\x18\x05 \x02(\x0e\x32\x17.motorcortex.StatusCode\"D\n\x0eRemoveGroupMsg\x12#\n\x06header\x18\x01 \x01(\x0b\x32\x13.motorcortex.Header\x12\r\n\x05\x61lias\x18\x02 \x02(\t\"D\n\x0fGetParameterMsg\x12#\n\x06header\x18\x01 \x01(\x0b\x32\x13.motorcortex.Header\x12\x0c\n\x04path\x18\x02 \x02(\t\"\x95\x01\n\x0cParameterMsg\x12\r\n\x05value\x18\x01 \x02(\x0c\x12#\n\x06header\x18\x02 \x01(\x0b\x32\x13.motorcortex.Header\x12(\n\x04info\x18\x03 \x01(\x0b\x32\x1a.motorcortex.ParameterInfo\x12\'\n\x06status\x18\x04 \x02(\x0e\x32\x17.motorcortex.StatusCode\"h\n\x13GetParameterListMsg\x12#\n\x06header\x18\x01 \x01(\x0b\x32\x13.motorcortex.Header\x12,\n\x06params\x18\x02 \x03(\x0b\x32\x1c.motorcortex.GetParameterMsg\"\x8b\x01\n\x10ParameterListMsg\x12#\n\x06header\x18\x01 \x01(\x0b\x32\x13.motorcortex.Header\x12)\n\x06params\x18\x02 \x03(\x0b\x32\x19.motorcortex.ParameterMsg\x12\'\n\x06status\x18\x03 \x02(\x0e\x32\x17.motorcortex.StatusCode\"\x81\x01\n\x0fSetParameterMsg\x12#\n\x06header\x18\x01 \x01(\x0b\x32\x13.motorcortex.Header\x12,\n\x06offset\x18\x02 \x01(\x0b\x32\x1c.motorcortex.ParameterOffset\x12\x0c\n\x04path\x18\x03 \x02(\t\x12\r\n\x05value\x18\x04 \x02(\x0c\"h\n\x13SetParameterListMsg\x12#\n\x06header\x18\x01 \x01(\x0b\x32\x13.motorcortex.Header\x12,\n\x06params\x18\x02 \x03(\x0b\x32\x1c.motorcortex.SetParameterMsg\"\x99\x01\n\x15OverwriteParameterMsg\x12#\n\x06header\x18\x01 \x01(\x0b\x32\x13.motorcortex.Header\x12,\n\x06offset\x18\x02 \x01(\x0b\x32\x1c.motorcortex.ParameterOffset\x12\x10\n\x08\x61\x63tivate\x18\x03 \x02(\x08\x12\x0c\n\x04path\x18\x04 \x02(\t\x12\r\n\x05value\x18\x05 \x02(\x0c\"H\n\x13ReleaseParameterMsg\x12#\n\x06header\x18\x01 \x01(\x0b\x32\x13.motorcortex.Header\x12\x0c\n\x04path\x18\x02 \x02(\t\"O\n\x07SaveMsg\x12#\n\x06header\x18\x01 \x01(\x0b\x32\x13.motorcortex.Header\x12\x0c\n\x04path\x18\x02 \x02(\t\x12\x11\n\tfile_name\x18\x03 \x02(\t\"O\n\x07LoadMsg\x12#\n\x06header\x18\x01 \x01(\x0b\x32\x13.motorcortex.Header\x12\x0c\n\x04path\x18\x02 \x02(\t\x12\x11\n\tfile_name\x18\x03 \x02(\t\"C\n\rConsoleCmdMsg\x12#\n\x06header\x18\x01 \x01(\x0b\x32\x13.motorcortex.Header\x12\r\n\x05value\x18\x02 \x02(\t\"b\n\x11\x43onsoleCmdListMsg\x12#\n\x06header\x18\x01 \x01(\x0b\x32\x13.motorcortex.Header\x12(\n\x04\x63mds\x18\x02 \x03(\x0b\x32\x1a.motorcortex.ConsoleCmdMsg*\xab\x02\n\x04Unit\x12\x12\n\x0eunit_undefined\x10\x00\x12\n\n\x06Length\x10\x0f\x12\x06\n\x02mm\x10\x01\x12\x05\n\x01m\x10\x02\x12\n\n\x05\x41ngle\x10\xf1\x01\x12\x07\n\x03rad\x10\x31\x12\x07\n\x03\x64\x65g\x10\x41\x12\t\n\x04Time\x10\xf2\x01\x12\x0b\n\x07nanosec\x10\x12\x12\x0c\n\x08microsec\x10\"\x12\x0c\n\x08millisec\x10\x32\x12\x07\n\x03sec\x10\x42\x12\x0b\n\x06Weight\x10\xf3\x01\x12\x08\n\x04gram\x10\x13\x12\x06\n\x02kg\x10#\x12\r\n\x08Velocity\x10\xf4\x01\x12\t\n\x05m_sec\x10\x14\x12\x0b\n\x07rad_sec\x10$\x12\x11\n\x0c\x41\x63\x63\x65leration\x10\xf5\x01\x12\n\n\x06m_sec2\x10\x15\x12\x0c\n\x08rad_sec2\x10%\x12\n\n\x05\x46orce\x10\xf6\x01\x12\x05\n\x01N\x10\x16\x12\x06\n\x02Nm\x10&\x12\x0b\n\x07percent\x10\x17*a\n\tUserGroup\x12\x18\n\x14user_group_undefined\x10\x00\x12\n\n\x06SYSTEM\x10\x01\x12\x11\n\rADMINISTRATOR\x10\x03\x12\x0c\n\x08OPERATOR\x10\x07\x12\r\n\x05GUEST\x10\xff\xff\xff\xff\x07*\xc4\x01\n\nPermission\x12\x18\n\x14permission_undefined\x10\x00\x12\x0e\n\tUSER_READ\x10\x80\x02\x12\x0f\n\nUSER_WRITE\x10\x80\x01\x12\x10\n\x0cUSER_EXECUTE\x10@\x12\x0e\n\nGROUP_READ\x10 \x12\x0f\n\x0bGROUP_WRITE\x10\x10\x12\x11\n\rGROUP_EXECUTE\x10\x08\x12\x0f\n\x0bOTHERS_READ\x10\x04\x12\x10\n\x0cOTHERS_WRITE\x10\x02\x12\x12\n\x0eOTHERS_EXECUTE\x10\x01*\xd4\x01\n\x08\x44\x61taType\x12\x17\n\x13\x64\x61ta_type_undefined\x10\x00\x12\x08\n\x04INT8\x10\x01\x12\t\n\x05UINT8\x10\x02\x12\t\n\x05INT16\x10\x03\x12\n\n\x06UINT16\x10\x04\x12\t\n\x05INT32\x10\x05\x12\n\n\x06UINT32\x10\x06\x12\t\n\x05INT64\x10\x07\x12\n\n\x06UINT64\x10\x08\x12\x08\n\x04\x42OOL\x10\t\x12\n\n\x05\x46LOAT\x10\x81\x02\x12\x0b\n\x06\x44OUBLE\x10\x82\x02\x12\t\n\x04\x43HAR\x10\x81\x04\x12\x0b\n\x06STRING\x10\x82\x04\x12\n\n\x05\x42YTES\x10\x99\t\x12\x0e\n\tUSER_TYPE\x10\x80\n*\x84\x01\n\rParameterType\x12\x18\n\x14param_type_undefined\x10\x00\x12\t\n\x05INPUT\x10\x01\x12\n\n\x06OUTPUT\x10\x10\x12\x0e\n\tPARAMETER\x10\x80\x02\x12\x17\n\x12PARAMETER_VOLATILE\x10\x81\x02\x12\x19\n\x14PARAMETER_PERSISTENT\x10\x82\x02*\x81\x02\n\nStatusCode\x12\x06\n\x02OK\x10\x00\x12\x12\n\x0eREAD_ONLY_MODE\x10\x01\x12\x0c\n\x06\x46\x41ILED\x10\x80\xfe\x03\x12\x15\n\x10\x46\x41ILED_TO_DECODE\x10\x80 \x12\x15\n\x10SUB_LIST_IS_FULL\x10\x80\"\x12\x19\n\x14WRONG_PARAMETER_PATH\x10\x80$\x12 \n\x1b\x46\x41ILED_TO_SET_REQUESTED_FRQ\x10\x80&\x12\x18\n\x13\x46\x41ILED_TO_OPEN_FILE\x10\x80(\x12\x13\n\x0eWRONG_PASSWORD\x10\x80\x42\x12\x17\n\x12USER_NOT_LOGGED_IN\x10\x80\x44\x12\x16\n\x11PERMISSION_DENIED\x10\x80\x46*v\n\nErrorLevel\x12\x19\n\x15\x65rror_level_undefined\x10\x00\x12\x08\n\x04INFO\x10\x01\x12\x0b\n\x07WARNING\x10\x02\x12\x14\n\x10\x46ORCED_DISENGAGE\x10\x03\x12\x0c\n\x08SHUTDOWN\x10\x04\x12\x12\n\x0e\x45MERGENCY_STOP\x10\x05*_\n\nOffsetType\x12\x19\n\x15offset_type_undefined\x10\x00\x12\x13\n\x0fOFFSET_ELEMENTS\x10\x01\x12\x10\n\x0cOFFSET_BYTES\x10\x02\x12\x0f\n\x0bOFFSET_BITS\x10\x03*<\n\rParameterFlag\x12\x12\n\x0eLINK_IS_ACTIVE\x10\x01\x12\x17\n\x13OVERWRITE_IS_ACTIVE\x10\x02')
)

_UNIT = _descriptor.EnumDescriptor(
  name='Unit',
  full_name='motorcortex.Unit',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='unit_undefined', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Length', index=1, number=15,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='mm', index=2, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='m', index=3, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Angle', index=4, number=241,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='rad', index=5, number=49,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='deg', index=6, number=65,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Time', index=7, number=242,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='nanosec', index=8, number=18,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='microsec', index=9, number=34,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='millisec', index=10, number=50,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='sec', index=11, number=66,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Weight', index=12, number=243,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='gram', index=13, number=19,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='kg', index=14, number=35,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Velocity', index=15, number=244,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='m_sec', index=16, number=20,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='rad_sec', index=17, number=36,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Acceleration', index=18, number=245,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='m_sec2', index=19, number=21,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='rad_sec2', index=20, number=37,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Force', index=21, number=246,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='N', index=22, number=22,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Nm', index=23, number=38,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='percent', index=24, number=23,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=3390,
  serialized_end=3689,
)
_sym_db.RegisterEnumDescriptor(_UNIT)

Unit = enum_type_wrapper.EnumTypeWrapper(_UNIT)
_USERGROUP = _descriptor.EnumDescriptor(
  name='UserGroup',
  full_name='motorcortex.UserGroup',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='user_group_undefined', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SYSTEM', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ADMINISTRATOR', index=2, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OPERATOR', index=3, number=7,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GUEST', index=4, number=2147483647,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=3691,
  serialized_end=3788,
)
_sym_db.RegisterEnumDescriptor(_USERGROUP)

UserGroup = enum_type_wrapper.EnumTypeWrapper(_USERGROUP)
_PERMISSION = _descriptor.EnumDescriptor(
  name='Permission',
  full_name='motorcortex.Permission',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='permission_undefined', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='USER_READ', index=1, number=256,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='USER_WRITE', index=2, number=128,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='USER_EXECUTE', index=3, number=64,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GROUP_READ', index=4, number=32,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GROUP_WRITE', index=5, number=16,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GROUP_EXECUTE', index=6, number=8,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OTHERS_READ', index=7, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OTHERS_WRITE', index=8, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OTHERS_EXECUTE', index=9, number=1,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=3791,
  serialized_end=3987,
)
_sym_db.RegisterEnumDescriptor(_PERMISSION)

Permission = enum_type_wrapper.EnumTypeWrapper(_PERMISSION)
_DATATYPE = _descriptor.EnumDescriptor(
  name='DataType',
  full_name='motorcortex.DataType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='data_type_undefined', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INT8', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UINT8', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INT16', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UINT16', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INT32', index=5, number=5,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UINT32', index=6, number=6,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INT64', index=7, number=7,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UINT64', index=8, number=8,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='BOOL', index=9, number=9,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FLOAT', index=10, number=257,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DOUBLE', index=11, number=258,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CHAR', index=12, number=513,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='STRING', index=13, number=514,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='BYTES', index=14, number=1177,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='USER_TYPE', index=15, number=1280,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=3990,
  serialized_end=4202,
)
_sym_db.RegisterEnumDescriptor(_DATATYPE)

DataType = enum_type_wrapper.EnumTypeWrapper(_DATATYPE)
_PARAMETERTYPE = _descriptor.EnumDescriptor(
  name='ParameterType',
  full_name='motorcortex.ParameterType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='param_type_undefined', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INPUT', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OUTPUT', index=2, number=16,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PARAMETER', index=3, number=256,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PARAMETER_VOLATILE', index=4, number=257,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PARAMETER_PERSISTENT', index=5, number=258,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=4205,
  serialized_end=4337,
)
_sym_db.RegisterEnumDescriptor(_PARAMETERTYPE)

ParameterType = enum_type_wrapper.EnumTypeWrapper(_PARAMETERTYPE)
_STATUSCODE = _descriptor.EnumDescriptor(
  name='StatusCode',
  full_name='motorcortex.StatusCode',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='OK', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='READ_ONLY_MODE', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FAILED', index=2, number=65280,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FAILED_TO_DECODE', index=3, number=4096,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUB_LIST_IS_FULL', index=4, number=4352,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WRONG_PARAMETER_PATH', index=5, number=4608,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FAILED_TO_SET_REQUESTED_FRQ', index=6, number=4864,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FAILED_TO_OPEN_FILE', index=7, number=5120,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WRONG_PASSWORD', index=8, number=8448,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='USER_NOT_LOGGED_IN', index=9, number=8704,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PERMISSION_DENIED', index=10, number=8960,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=4340,
  serialized_end=4597,
)
_sym_db.RegisterEnumDescriptor(_STATUSCODE)

StatusCode = enum_type_wrapper.EnumTypeWrapper(_STATUSCODE)
_ERRORLEVEL = _descriptor.EnumDescriptor(
  name='ErrorLevel',
  full_name='motorcortex.ErrorLevel',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='error_level_undefined', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INFO', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WARNING', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FORCED_DISENGAGE', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SHUTDOWN', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='EMERGENCY_STOP', index=5, number=5,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=4599,
  serialized_end=4717,
)
_sym_db.RegisterEnumDescriptor(_ERRORLEVEL)

ErrorLevel = enum_type_wrapper.EnumTypeWrapper(_ERRORLEVEL)
_OFFSETTYPE = _descriptor.EnumDescriptor(
  name='OffsetType',
  full_name='motorcortex.OffsetType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='offset_type_undefined', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OFFSET_ELEMENTS', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OFFSET_BYTES', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OFFSET_BITS', index=3, number=3,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=4719,
  serialized_end=4814,
)
_sym_db.RegisterEnumDescriptor(_OFFSETTYPE)

OffsetType = enum_type_wrapper.EnumTypeWrapper(_OFFSETTYPE)
_PARAMETERFLAG = _descriptor.EnumDescriptor(
  name='ParameterFlag',
  full_name='motorcortex.ParameterFlag',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='LINK_IS_ACTIVE', index=0, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OVERWRITE_IS_ACTIVE', index=1, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=4816,
  serialized_end=4876,
)
_sym_db.RegisterEnumDescriptor(_PARAMETERFLAG)

ParameterFlag = enum_type_wrapper.EnumTypeWrapper(_PARAMETERFLAG)
unit_undefined = 0
Length = 15
mm = 1
m = 2
Angle = 241
rad = 49
deg = 65
Time = 242
nanosec = 18
microsec = 34
millisec = 50
sec = 66
Weight = 243
gram = 19
kg = 35
Velocity = 244
m_sec = 20
rad_sec = 36
Acceleration = 245
m_sec2 = 21
rad_sec2 = 37
Force = 246
N = 22
Nm = 38
percent = 23
user_group_undefined = 0
SYSTEM = 1
ADMINISTRATOR = 3
OPERATOR = 7
GUEST = 2147483647
permission_undefined = 0
USER_READ = 256
USER_WRITE = 128
USER_EXECUTE = 64
GROUP_READ = 32
GROUP_WRITE = 16
GROUP_EXECUTE = 8
OTHERS_READ = 4
OTHERS_WRITE = 2
OTHERS_EXECUTE = 1
data_type_undefined = 0
INT8 = 1
UINT8 = 2
INT16 = 3
UINT16 = 4
INT32 = 5
UINT32 = 6
INT64 = 7
UINT64 = 8
BOOL = 9
FLOAT = 257
DOUBLE = 258
CHAR = 513
STRING = 514
BYTES = 1177
USER_TYPE = 1280
param_type_undefined = 0
INPUT = 1
OUTPUT = 16
PARAMETER = 256
PARAMETER_VOLATILE = 257
PARAMETER_PERSISTENT = 258
OK = 0
READ_ONLY_MODE = 1
FAILED = 65280
FAILED_TO_DECODE = 4096
SUB_LIST_IS_FULL = 4352
WRONG_PARAMETER_PATH = 4608
FAILED_TO_SET_REQUESTED_FRQ = 4864
FAILED_TO_OPEN_FILE = 5120
WRONG_PASSWORD = 8448
USER_NOT_LOGGED_IN = 8704
PERMISSION_DENIED = 8960
error_level_undefined = 0
INFO = 1
WARNING = 2
FORCED_DISENGAGE = 3
SHUTDOWN = 4
EMERGENCY_STOP = 5
offset_type_undefined = 0
OFFSET_ELEMENTS = 1
OFFSET_BYTES = 2
OFFSET_BITS = 3
LINK_IS_ACTIVE = 1
OVERWRITE_IS_ACTIVE = 2



_PARAMETEROFFSET = _descriptor.Descriptor(
  name='ParameterOffset',
  full_name='motorcortex.ParameterOffset',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='motorcortex.ParameterOffset.type', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=True, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='offset', full_name='motorcortex.ParameterOffset.offset', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='length', full_name='motorcortex.ParameterOffset.length', index=2,
      number=3, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=34,
  serialized_end=139,
)


_ERROR = _descriptor.Descriptor(
  name='Error',
  full_name='motorcortex.Error',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='motorcortex.Error.timestamp', index=0,
      number=1, type=6, cpp_type=4, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='error_number', full_name='motorcortex.Error.error_number', index=1,
      number=2, type=7, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='error_level', full_name='motorcortex.Error.error_level', index=2,
      number=3, type=7, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='subsystem', full_name='motorcortex.Error.subsystem', index=3,
      number=4, type=7, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='info', full_name='motorcortex.Error.info', index=4,
      number=5, type=7, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=141,
  serialized_end=243,
)


_ERRORLIST = _descriptor.Descriptor(
  name='ErrorList',
  full_name='motorcortex.ErrorList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='errors', full_name='motorcortex.ErrorList.errors', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='number_of_errors', full_name='motorcortex.ErrorList.number_of_errors', index=1,
      number=2, type=7, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='update_counter', full_name='motorcortex.ErrorList.update_counter', index=2,
      number=3, type=7, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=245,
  serialized_end=342,
)


_PARAMETERINFO = _descriptor.Descriptor(
  name='ParameterInfo',
  full_name='motorcortex.ParameterInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='motorcortex.ParameterInfo.id', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='data_type', full_name='motorcortex.ParameterInfo.data_type', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='data_size', full_name='motorcortex.ParameterInfo.data_size', index=2,
      number=3, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='number_of_elements', full_name='motorcortex.ParameterInfo.number_of_elements', index=3,
      number=4, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='flags', full_name='motorcortex.ParameterInfo.flags', index=4,
      number=5, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='permissions', full_name='motorcortex.ParameterInfo.permissions', index=5,
      number=6, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='param_type', full_name='motorcortex.ParameterInfo.param_type', index=6,
      number=7, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='group_id', full_name='motorcortex.ParameterInfo.group_id', index=7,
      number=8, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='unit', full_name='motorcortex.ParameterInfo.unit', index=8,
      number=9, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='path', full_name='motorcortex.ParameterInfo.path', index=9,
      number=10, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=345,
  serialized_end=611,
)


_GROUPPARAMETERINFO = _descriptor.Descriptor(
  name='GroupParameterInfo',
  full_name='motorcortex.GroupParameterInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='index', full_name='motorcortex.GroupParameterInfo.index', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='offset', full_name='motorcortex.GroupParameterInfo.offset', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='size', full_name='motorcortex.GroupParameterInfo.size', index=2,
      number=3, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='info', full_name='motorcortex.GroupParameterInfo.info', index=3,
      number=4, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='motorcortex.GroupParameterInfo.status', index=4,
      number=5, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=614,
  serialized_end=762,
)


_HEADER = _descriptor.Descriptor(
  name='Header',
  full_name='motorcortex.Header',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='frameCounter', full_name='motorcortex.Header.frameCounter', index=0,
      number=1, type=7, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='motorcortex.Header.timestamp', index=1,
      number=2, type=6, cpp_type=4, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=764,
  serialized_end=813,
)


_GROUPMSG = _descriptor.Descriptor(
  name='GroupMsg',
  full_name='motorcortex.GroupMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='motorcortex.GroupMsg.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='params', full_name='motorcortex.GroupMsg.params', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=815,
  serialized_end=905,
)


_STATUSMSG = _descriptor.Descriptor(
  name='StatusMsg',
  full_name='motorcortex.StatusMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='motorcortex.StatusMsg.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='motorcortex.StatusMsg.status', index=1,
      number=2, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=907,
  serialized_end=996,
)


_LOGINMSG = _descriptor.Descriptor(
  name='LoginMsg',
  full_name='motorcortex.LoginMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='motorcortex.LoginMsg.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='login', full_name='motorcortex.LoginMsg.login', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='password', full_name='motorcortex.LoginMsg.password', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=998,
  serialized_end=1078,
)


_GETSESSIONTOKENMSG = _descriptor.Descriptor(
  name='GetSessionTokenMsg',
  full_name='motorcortex.GetSessionTokenMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='motorcortex.GetSessionTokenMsg.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1080,
  serialized_end=1137,
)


_SESSIONTOKENMSG = _descriptor.Descriptor(
  name='SessionTokenMsg',
  full_name='motorcortex.SessionTokenMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='motorcortex.SessionTokenMsg.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='token', full_name='motorcortex.SessionTokenMsg.token', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='motorcortex.SessionTokenMsg.status', index=2,
      number=3, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1139,
  serialized_end=1249,
)


_RESTORESESSIONMSG = _descriptor.Descriptor(
  name='RestoreSessionMsg',
  full_name='motorcortex.RestoreSessionMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='motorcortex.RestoreSessionMsg.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='token', full_name='motorcortex.RestoreSessionMsg.token', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1251,
  serialized_end=1322,
)


_LOGOUTMSG = _descriptor.Descriptor(
  name='LogoutMsg',
  full_name='motorcortex.LogoutMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='motorcortex.LogoutMsg.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1324,
  serialized_end=1372,
)


_GETPARAMETERTREEMSG = _descriptor.Descriptor(
  name='GetParameterTreeMsg',
  full_name='motorcortex.GetParameterTreeMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='motorcortex.GetParameterTreeMsg.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1374,
  serialized_end=1432,
)


_PARAMETERTREEMSG = _descriptor.Descriptor(
  name='ParameterTreeMsg',
  full_name='motorcortex.ParameterTreeMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='motorcortex.ParameterTreeMsg.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='params', full_name='motorcortex.ParameterTreeMsg.params', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hash', full_name='motorcortex.ParameterTreeMsg.hash', index=2,
      number=3, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='motorcortex.ParameterTreeMsg.status', index=3,
      number=4, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1435,
  serialized_end=1589,
)


_GETPARAMETERTREEHASHMSG = _descriptor.Descriptor(
  name='GetParameterTreeHashMsg',
  full_name='motorcortex.GetParameterTreeHashMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='motorcortex.GetParameterTreeHashMsg.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1591,
  serialized_end=1653,
)


_PARAMETERTREEHASHMSG = _descriptor.Descriptor(
  name='ParameterTreeHashMsg',
  full_name='motorcortex.ParameterTreeHashMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='motorcortex.ParameterTreeHashMsg.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hash', full_name='motorcortex.ParameterTreeHashMsg.hash', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='motorcortex.ParameterTreeHashMsg.status', index=2,
      number=3, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1655,
  serialized_end=1769,
)


_CREATEGROUPMSG = _descriptor.Descriptor(
  name='CreateGroupMsg',
  full_name='motorcortex.CreateGroupMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='motorcortex.CreateGroupMsg.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='frq_divider', full_name='motorcortex.CreateGroupMsg.frq_divider', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='alias', full_name='motorcortex.CreateGroupMsg.alias', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='paths', full_name='motorcortex.CreateGroupMsg.paths', index=3,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1771,
  serialized_end=1875,
)


_GROUPSTATUSMSG = _descriptor.Descriptor(
  name='GroupStatusMsg',
  full_name='motorcortex.GroupStatusMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='motorcortex.GroupStatusMsg.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='id', full_name='motorcortex.GroupStatusMsg.id', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='alias', full_name='motorcortex.GroupStatusMsg.alias', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='params', full_name='motorcortex.GroupStatusMsg.params', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='motorcortex.GroupStatusMsg.status', index=4,
      number=5, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1878,
  serialized_end=2048,
)


_REMOVEGROUPMSG = _descriptor.Descriptor(
  name='RemoveGroupMsg',
  full_name='motorcortex.RemoveGroupMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='motorcortex.RemoveGroupMsg.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='alias', full_name='motorcortex.RemoveGroupMsg.alias', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2050,
  serialized_end=2118,
)


_GETPARAMETERMSG = _descriptor.Descriptor(
  name='GetParameterMsg',
  full_name='motorcortex.GetParameterMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='motorcortex.GetParameterMsg.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='path', full_name='motorcortex.GetParameterMsg.path', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2120,
  serialized_end=2188,
)


_PARAMETERMSG = _descriptor.Descriptor(
  name='ParameterMsg',
  full_name='motorcortex.ParameterMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='motorcortex.ParameterMsg.value', index=0,
      number=1, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='header', full_name='motorcortex.ParameterMsg.header', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='info', full_name='motorcortex.ParameterMsg.info', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='motorcortex.ParameterMsg.status', index=3,
      number=4, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2191,
  serialized_end=2340,
)


_GETPARAMETERLISTMSG = _descriptor.Descriptor(
  name='GetParameterListMsg',
  full_name='motorcortex.GetParameterListMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='motorcortex.GetParameterListMsg.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='params', full_name='motorcortex.GetParameterListMsg.params', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2342,
  serialized_end=2446,
)


_PARAMETERLISTMSG = _descriptor.Descriptor(
  name='ParameterListMsg',
  full_name='motorcortex.ParameterListMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='motorcortex.ParameterListMsg.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='params', full_name='motorcortex.ParameterListMsg.params', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='motorcortex.ParameterListMsg.status', index=2,
      number=3, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2449,
  serialized_end=2588,
)


_SETPARAMETERMSG = _descriptor.Descriptor(
  name='SetParameterMsg',
  full_name='motorcortex.SetParameterMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='motorcortex.SetParameterMsg.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='offset', full_name='motorcortex.SetParameterMsg.offset', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='path', full_name='motorcortex.SetParameterMsg.path', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='motorcortex.SetParameterMsg.value', index=3,
      number=4, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2591,
  serialized_end=2720,
)


_SETPARAMETERLISTMSG = _descriptor.Descriptor(
  name='SetParameterListMsg',
  full_name='motorcortex.SetParameterListMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='motorcortex.SetParameterListMsg.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='params', full_name='motorcortex.SetParameterListMsg.params', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2722,
  serialized_end=2826,
)


_OVERWRITEPARAMETERMSG = _descriptor.Descriptor(
  name='OverwriteParameterMsg',
  full_name='motorcortex.OverwriteParameterMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='motorcortex.OverwriteParameterMsg.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='offset', full_name='motorcortex.OverwriteParameterMsg.offset', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='activate', full_name='motorcortex.OverwriteParameterMsg.activate', index=2,
      number=3, type=8, cpp_type=7, label=2,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='path', full_name='motorcortex.OverwriteParameterMsg.path', index=3,
      number=4, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='motorcortex.OverwriteParameterMsg.value', index=4,
      number=5, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2829,
  serialized_end=2982,
)


_RELEASEPARAMETERMSG = _descriptor.Descriptor(
  name='ReleaseParameterMsg',
  full_name='motorcortex.ReleaseParameterMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='motorcortex.ReleaseParameterMsg.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='path', full_name='motorcortex.ReleaseParameterMsg.path', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2984,
  serialized_end=3056,
)


_SAVEMSG = _descriptor.Descriptor(
  name='SaveMsg',
  full_name='motorcortex.SaveMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='motorcortex.SaveMsg.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='path', full_name='motorcortex.SaveMsg.path', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='file_name', full_name='motorcortex.SaveMsg.file_name', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3058,
  serialized_end=3137,
)


_LOADMSG = _descriptor.Descriptor(
  name='LoadMsg',
  full_name='motorcortex.LoadMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='motorcortex.LoadMsg.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='path', full_name='motorcortex.LoadMsg.path', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='file_name', full_name='motorcortex.LoadMsg.file_name', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3139,
  serialized_end=3218,
)


_CONSOLECMDMSG = _descriptor.Descriptor(
  name='ConsoleCmdMsg',
  full_name='motorcortex.ConsoleCmdMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='motorcortex.ConsoleCmdMsg.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='motorcortex.ConsoleCmdMsg.value', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3220,
  serialized_end=3287,
)


_CONSOLECMDLISTMSG = _descriptor.Descriptor(
  name='ConsoleCmdListMsg',
  full_name='motorcortex.ConsoleCmdListMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='motorcortex.ConsoleCmdListMsg.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cmds', full_name='motorcortex.ConsoleCmdListMsg.cmds', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3289,
  serialized_end=3387,
)

_PARAMETEROFFSET.fields_by_name['type'].enum_type = _OFFSETTYPE
_ERRORLIST.fields_by_name['errors'].message_type = _ERROR
_PARAMETERINFO.fields_by_name['param_type'].enum_type = _PARAMETERTYPE
_PARAMETERINFO.fields_by_name['group_id'].enum_type = _USERGROUP
_PARAMETERINFO.fields_by_name['unit'].enum_type = _UNIT
_GROUPPARAMETERINFO.fields_by_name['info'].message_type = _PARAMETERINFO
_GROUPPARAMETERINFO.fields_by_name['status'].enum_type = _STATUSCODE
_GROUPMSG.fields_by_name['header'].message_type = _HEADER
_GROUPMSG.fields_by_name['params'].message_type = _PARAMETERMSG
_STATUSMSG.fields_by_name['header'].message_type = _HEADER
_STATUSMSG.fields_by_name['status'].enum_type = _STATUSCODE
_LOGINMSG.fields_by_name['header'].message_type = _HEADER
_GETSESSIONTOKENMSG.fields_by_name['header'].message_type = _HEADER
_SESSIONTOKENMSG.fields_by_name['header'].message_type = _HEADER
_SESSIONTOKENMSG.fields_by_name['status'].enum_type = _STATUSCODE
_RESTORESESSIONMSG.fields_by_name['header'].message_type = _HEADER
_LOGOUTMSG.fields_by_name['header'].message_type = _HEADER
_GETPARAMETERTREEMSG.fields_by_name['header'].message_type = _HEADER
_PARAMETERTREEMSG.fields_by_name['header'].message_type = _HEADER
_PARAMETERTREEMSG.fields_by_name['params'].message_type = _PARAMETERINFO
_PARAMETERTREEMSG.fields_by_name['status'].enum_type = _STATUSCODE
_GETPARAMETERTREEHASHMSG.fields_by_name['header'].message_type = _HEADER
_PARAMETERTREEHASHMSG.fields_by_name['header'].message_type = _HEADER
_PARAMETERTREEHASHMSG.fields_by_name['status'].enum_type = _STATUSCODE
_CREATEGROUPMSG.fields_by_name['header'].message_type = _HEADER
_GROUPSTATUSMSG.fields_by_name['header'].message_type = _HEADER
_GROUPSTATUSMSG.fields_by_name['params'].message_type = _GROUPPARAMETERINFO
_GROUPSTATUSMSG.fields_by_name['status'].enum_type = _STATUSCODE
_REMOVEGROUPMSG.fields_by_name['header'].message_type = _HEADER
_GETPARAMETERMSG.fields_by_name['header'].message_type = _HEADER
_PARAMETERMSG.fields_by_name['header'].message_type = _HEADER
_PARAMETERMSG.fields_by_name['info'].message_type = _PARAMETERINFO
_PARAMETERMSG.fields_by_name['status'].enum_type = _STATUSCODE
_GETPARAMETERLISTMSG.fields_by_name['header'].message_type = _HEADER
_GETPARAMETERLISTMSG.fields_by_name['params'].message_type = _GETPARAMETERMSG
_PARAMETERLISTMSG.fields_by_name['header'].message_type = _HEADER
_PARAMETERLISTMSG.fields_by_name['params'].message_type = _PARAMETERMSG
_PARAMETERLISTMSG.fields_by_name['status'].enum_type = _STATUSCODE
_SETPARAMETERMSG.fields_by_name['header'].message_type = _HEADER
_SETPARAMETERMSG.fields_by_name['offset'].message_type = _PARAMETEROFFSET
_SETPARAMETERLISTMSG.fields_by_name['header'].message_type = _HEADER
_SETPARAMETERLISTMSG.fields_by_name['params'].message_type = _SETPARAMETERMSG
_OVERWRITEPARAMETERMSG.fields_by_name['header'].message_type = _HEADER
_OVERWRITEPARAMETERMSG.fields_by_name['offset'].message_type = _PARAMETEROFFSET
_RELEASEPARAMETERMSG.fields_by_name['header'].message_type = _HEADER
_SAVEMSG.fields_by_name['header'].message_type = _HEADER
_LOADMSG.fields_by_name['header'].message_type = _HEADER
_CONSOLECMDMSG.fields_by_name['header'].message_type = _HEADER
_CONSOLECMDLISTMSG.fields_by_name['header'].message_type = _HEADER
_CONSOLECMDLISTMSG.fields_by_name['cmds'].message_type = _CONSOLECMDMSG
DESCRIPTOR.message_types_by_name['ParameterOffset'] = _PARAMETEROFFSET
DESCRIPTOR.message_types_by_name['Error'] = _ERROR
DESCRIPTOR.message_types_by_name['ErrorList'] = _ERRORLIST
DESCRIPTOR.message_types_by_name['ParameterInfo'] = _PARAMETERINFO
DESCRIPTOR.message_types_by_name['GroupParameterInfo'] = _GROUPPARAMETERINFO
DESCRIPTOR.message_types_by_name['Header'] = _HEADER
DESCRIPTOR.message_types_by_name['GroupMsg'] = _GROUPMSG
DESCRIPTOR.message_types_by_name['StatusMsg'] = _STATUSMSG
DESCRIPTOR.message_types_by_name['LoginMsg'] = _LOGINMSG
DESCRIPTOR.message_types_by_name['GetSessionTokenMsg'] = _GETSESSIONTOKENMSG
DESCRIPTOR.message_types_by_name['SessionTokenMsg'] = _SESSIONTOKENMSG
DESCRIPTOR.message_types_by_name['RestoreSessionMsg'] = _RESTORESESSIONMSG
DESCRIPTOR.message_types_by_name['LogoutMsg'] = _LOGOUTMSG
DESCRIPTOR.message_types_by_name['GetParameterTreeMsg'] = _GETPARAMETERTREEMSG
DESCRIPTOR.message_types_by_name['ParameterTreeMsg'] = _PARAMETERTREEMSG
DESCRIPTOR.message_types_by_name['GetParameterTreeHashMsg'] = _GETPARAMETERTREEHASHMSG
DESCRIPTOR.message_types_by_name['ParameterTreeHashMsg'] = _PARAMETERTREEHASHMSG
DESCRIPTOR.message_types_by_name['CreateGroupMsg'] = _CREATEGROUPMSG
DESCRIPTOR.message_types_by_name['GroupStatusMsg'] = _GROUPSTATUSMSG
DESCRIPTOR.message_types_by_name['RemoveGroupMsg'] = _REMOVEGROUPMSG
DESCRIPTOR.message_types_by_name['GetParameterMsg'] = _GETPARAMETERMSG
DESCRIPTOR.message_types_by_name['ParameterMsg'] = _PARAMETERMSG
DESCRIPTOR.message_types_by_name['GetParameterListMsg'] = _GETPARAMETERLISTMSG
DESCRIPTOR.message_types_by_name['ParameterListMsg'] = _PARAMETERLISTMSG
DESCRIPTOR.message_types_by_name['SetParameterMsg'] = _SETPARAMETERMSG
DESCRIPTOR.message_types_by_name['SetParameterListMsg'] = _SETPARAMETERLISTMSG
DESCRIPTOR.message_types_by_name['OverwriteParameterMsg'] = _OVERWRITEPARAMETERMSG
DESCRIPTOR.message_types_by_name['ReleaseParameterMsg'] = _RELEASEPARAMETERMSG
DESCRIPTOR.message_types_by_name['SaveMsg'] = _SAVEMSG
DESCRIPTOR.message_types_by_name['LoadMsg'] = _LOADMSG
DESCRIPTOR.message_types_by_name['ConsoleCmdMsg'] = _CONSOLECMDMSG
DESCRIPTOR.message_types_by_name['ConsoleCmdListMsg'] = _CONSOLECMDLISTMSG
DESCRIPTOR.enum_types_by_name['Unit'] = _UNIT
DESCRIPTOR.enum_types_by_name['UserGroup'] = _USERGROUP
DESCRIPTOR.enum_types_by_name['Permission'] = _PERMISSION
DESCRIPTOR.enum_types_by_name['DataType'] = _DATATYPE
DESCRIPTOR.enum_types_by_name['ParameterType'] = _PARAMETERTYPE
DESCRIPTOR.enum_types_by_name['StatusCode'] = _STATUSCODE
DESCRIPTOR.enum_types_by_name['ErrorLevel'] = _ERRORLEVEL
DESCRIPTOR.enum_types_by_name['OffsetType'] = _OFFSETTYPE
DESCRIPTOR.enum_types_by_name['ParameterFlag'] = _PARAMETERFLAG
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ParameterOffset = _reflection.GeneratedProtocolMessageType('ParameterOffset', (_message.Message,), dict(
  DESCRIPTOR = _PARAMETEROFFSET,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.ParameterOffset)
  ))
_sym_db.RegisterMessage(ParameterOffset)

Error = _reflection.GeneratedProtocolMessageType('Error', (_message.Message,), dict(
  DESCRIPTOR = _ERROR,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.Error)
  ))
_sym_db.RegisterMessage(Error)

ErrorList = _reflection.GeneratedProtocolMessageType('ErrorList', (_message.Message,), dict(
  DESCRIPTOR = _ERRORLIST,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.ErrorList)
  ))
_sym_db.RegisterMessage(ErrorList)

ParameterInfo = _reflection.GeneratedProtocolMessageType('ParameterInfo', (_message.Message,), dict(
  DESCRIPTOR = _PARAMETERINFO,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.ParameterInfo)
  ))
_sym_db.RegisterMessage(ParameterInfo)

GroupParameterInfo = _reflection.GeneratedProtocolMessageType('GroupParameterInfo', (_message.Message,), dict(
  DESCRIPTOR = _GROUPPARAMETERINFO,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.GroupParameterInfo)
  ))
_sym_db.RegisterMessage(GroupParameterInfo)

Header = _reflection.GeneratedProtocolMessageType('Header', (_message.Message,), dict(
  DESCRIPTOR = _HEADER,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.Header)
  ))
_sym_db.RegisterMessage(Header)

GroupMsg = _reflection.GeneratedProtocolMessageType('GroupMsg', (_message.Message,), dict(
  DESCRIPTOR = _GROUPMSG,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.GroupMsg)
  ))
_sym_db.RegisterMessage(GroupMsg)

StatusMsg = _reflection.GeneratedProtocolMessageType('StatusMsg', (_message.Message,), dict(
  DESCRIPTOR = _STATUSMSG,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.StatusMsg)
  ))
_sym_db.RegisterMessage(StatusMsg)

LoginMsg = _reflection.GeneratedProtocolMessageType('LoginMsg', (_message.Message,), dict(
  DESCRIPTOR = _LOGINMSG,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.LoginMsg)
  ))
_sym_db.RegisterMessage(LoginMsg)

GetSessionTokenMsg = _reflection.GeneratedProtocolMessageType('GetSessionTokenMsg', (_message.Message,), dict(
  DESCRIPTOR = _GETSESSIONTOKENMSG,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.GetSessionTokenMsg)
  ))
_sym_db.RegisterMessage(GetSessionTokenMsg)

SessionTokenMsg = _reflection.GeneratedProtocolMessageType('SessionTokenMsg', (_message.Message,), dict(
  DESCRIPTOR = _SESSIONTOKENMSG,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.SessionTokenMsg)
  ))
_sym_db.RegisterMessage(SessionTokenMsg)

RestoreSessionMsg = _reflection.GeneratedProtocolMessageType('RestoreSessionMsg', (_message.Message,), dict(
  DESCRIPTOR = _RESTORESESSIONMSG,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.RestoreSessionMsg)
  ))
_sym_db.RegisterMessage(RestoreSessionMsg)

LogoutMsg = _reflection.GeneratedProtocolMessageType('LogoutMsg', (_message.Message,), dict(
  DESCRIPTOR = _LOGOUTMSG,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.LogoutMsg)
  ))
_sym_db.RegisterMessage(LogoutMsg)

GetParameterTreeMsg = _reflection.GeneratedProtocolMessageType('GetParameterTreeMsg', (_message.Message,), dict(
  DESCRIPTOR = _GETPARAMETERTREEMSG,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.GetParameterTreeMsg)
  ))
_sym_db.RegisterMessage(GetParameterTreeMsg)

ParameterTreeMsg = _reflection.GeneratedProtocolMessageType('ParameterTreeMsg', (_message.Message,), dict(
  DESCRIPTOR = _PARAMETERTREEMSG,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.ParameterTreeMsg)
  ))
_sym_db.RegisterMessage(ParameterTreeMsg)

GetParameterTreeHashMsg = _reflection.GeneratedProtocolMessageType('GetParameterTreeHashMsg', (_message.Message,), dict(
  DESCRIPTOR = _GETPARAMETERTREEHASHMSG,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.GetParameterTreeHashMsg)
  ))
_sym_db.RegisterMessage(GetParameterTreeHashMsg)

ParameterTreeHashMsg = _reflection.GeneratedProtocolMessageType('ParameterTreeHashMsg', (_message.Message,), dict(
  DESCRIPTOR = _PARAMETERTREEHASHMSG,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.ParameterTreeHashMsg)
  ))
_sym_db.RegisterMessage(ParameterTreeHashMsg)

CreateGroupMsg = _reflection.GeneratedProtocolMessageType('CreateGroupMsg', (_message.Message,), dict(
  DESCRIPTOR = _CREATEGROUPMSG,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.CreateGroupMsg)
  ))
_sym_db.RegisterMessage(CreateGroupMsg)

GroupStatusMsg = _reflection.GeneratedProtocolMessageType('GroupStatusMsg', (_message.Message,), dict(
  DESCRIPTOR = _GROUPSTATUSMSG,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.GroupStatusMsg)
  ))
_sym_db.RegisterMessage(GroupStatusMsg)

RemoveGroupMsg = _reflection.GeneratedProtocolMessageType('RemoveGroupMsg', (_message.Message,), dict(
  DESCRIPTOR = _REMOVEGROUPMSG,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.RemoveGroupMsg)
  ))
_sym_db.RegisterMessage(RemoveGroupMsg)

GetParameterMsg = _reflection.GeneratedProtocolMessageType('GetParameterMsg', (_message.Message,), dict(
  DESCRIPTOR = _GETPARAMETERMSG,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.GetParameterMsg)
  ))
_sym_db.RegisterMessage(GetParameterMsg)

ParameterMsg = _reflection.GeneratedProtocolMessageType('ParameterMsg', (_message.Message,), dict(
  DESCRIPTOR = _PARAMETERMSG,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.ParameterMsg)
  ))
_sym_db.RegisterMessage(ParameterMsg)

GetParameterListMsg = _reflection.GeneratedProtocolMessageType('GetParameterListMsg', (_message.Message,), dict(
  DESCRIPTOR = _GETPARAMETERLISTMSG,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.GetParameterListMsg)
  ))
_sym_db.RegisterMessage(GetParameterListMsg)

ParameterListMsg = _reflection.GeneratedProtocolMessageType('ParameterListMsg', (_message.Message,), dict(
  DESCRIPTOR = _PARAMETERLISTMSG,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.ParameterListMsg)
  ))
_sym_db.RegisterMessage(ParameterListMsg)

SetParameterMsg = _reflection.GeneratedProtocolMessageType('SetParameterMsg', (_message.Message,), dict(
  DESCRIPTOR = _SETPARAMETERMSG,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.SetParameterMsg)
  ))
_sym_db.RegisterMessage(SetParameterMsg)

SetParameterListMsg = _reflection.GeneratedProtocolMessageType('SetParameterListMsg', (_message.Message,), dict(
  DESCRIPTOR = _SETPARAMETERLISTMSG,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.SetParameterListMsg)
  ))
_sym_db.RegisterMessage(SetParameterListMsg)

OverwriteParameterMsg = _reflection.GeneratedProtocolMessageType('OverwriteParameterMsg', (_message.Message,), dict(
  DESCRIPTOR = _OVERWRITEPARAMETERMSG,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.OverwriteParameterMsg)
  ))
_sym_db.RegisterMessage(OverwriteParameterMsg)

ReleaseParameterMsg = _reflection.GeneratedProtocolMessageType('ReleaseParameterMsg', (_message.Message,), dict(
  DESCRIPTOR = _RELEASEPARAMETERMSG,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.ReleaseParameterMsg)
  ))
_sym_db.RegisterMessage(ReleaseParameterMsg)

SaveMsg = _reflection.GeneratedProtocolMessageType('SaveMsg', (_message.Message,), dict(
  DESCRIPTOR = _SAVEMSG,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.SaveMsg)
  ))
_sym_db.RegisterMessage(SaveMsg)

LoadMsg = _reflection.GeneratedProtocolMessageType('LoadMsg', (_message.Message,), dict(
  DESCRIPTOR = _LOADMSG,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.LoadMsg)
  ))
_sym_db.RegisterMessage(LoadMsg)

ConsoleCmdMsg = _reflection.GeneratedProtocolMessageType('ConsoleCmdMsg', (_message.Message,), dict(
  DESCRIPTOR = _CONSOLECMDMSG,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.ConsoleCmdMsg)
  ))
_sym_db.RegisterMessage(ConsoleCmdMsg)

ConsoleCmdListMsg = _reflection.GeneratedProtocolMessageType('ConsoleCmdListMsg', (_message.Message,), dict(
  DESCRIPTOR = _CONSOLECMDLISTMSG,
  __module__ = 'motorcortex_pb2'
  # @@protoc_insertion_point(class_scope:motorcortex.ConsoleCmdListMsg)
  ))
_sym_db.RegisterMessage(ConsoleCmdListMsg)


# @@protoc_insertion_point(module_scope)



#!/usr/bin/python3

#
#   Developer : Alexey Zakharov (alexey.zakharov@vectioneer.com)
#   All rights reserved. Copyright (c) 2018 VECTIONEER.
#
import logging
import tempfile
import unittest
import motorcortex
import time

SERVER = 'localhost'


class DefaultMessageTypesTest(unittest.TestCase):

    def test(self):
        motorcortex_types = motorcortex.MessageTypes()
        # check if namespaces exist
        self.assertTrue(motorcortex_types.getNamespace("motorcortex"))
        motorcortex_msg = motorcortex_types.motorcortex()
        self.assertTrue(motorcortex_msg)
        # load user messages with the same module name
        msg_list = motorcortex_types.load([{'proto': './motorcortex-msg/motorcortex_pb2.py',
                                            'hash': './motorcortex-msg/motorcortex_hash.json'}])
        # check if only a single motorcortex exists
        self.assertTrue(len(msg_list) == 1)
        # check if namespaces exist
        self.assertTrue(motorcortex_types.getNamespace("motorcortex"))


class MessageTypesTest(unittest.TestCase):

    def test(self):
        motorcortex_types = motorcortex.MessageTypes()
        motorcortex_msg = motorcortex_types.motorcortex()
        motionsl_msg, = motorcortex_types.load(
            [{'proto': './motorcortex-msg/motionSL_pb2.py', 'hash': './motorcortex-msg/motionSL_hash.json'}])

        # check if namespaces exist
        self.assertTrue(motorcortex_types.getNamespace("motorcortex"))
        self.assertTrue(motorcortex_types.getNamespace("motion_spec"))

        # check if enums are loaded correctly
        self.assertTrue(hasattr(motorcortex_msg, 'OK'))
        # check if types are loaded
        self.assertTrue(hasattr(motorcortex_msg, 'ParameterMsg'))


class OpenRequestConnection(unittest.TestCase):

    def test(self):
        # Open request connection
        motorcortex_types = motorcortex.MessageTypes()
        parameter_tree = motorcortex.ParameterTree()
        req = motorcortex.Request(motorcortex_types, parameter_tree)
        if req.connect("ws://%s:5558" % SERVER).get(1000):
            print("Request connection is etablished")
        else:
            print("Failed to establish Request connection")
            self.assertTrue(False)

        req.close()


class OpenSubscribeConnection(unittest.TestCase):

    def test(self):
        # Open subscribe connection
        motorcortex_types = motorcortex.MessageTypes()
        motorcortex_msg = motorcortex_types.motorcortex()
        parameter_tree = motorcortex.ParameterTree()
        req = motorcortex.Request(motorcortex_types, parameter_tree)
        req.connect("ws://%s:5558" % SERVER)
        sub = motorcortex.Subscribe(req, motorcortex_types)
        if sub.connect("ws://%s:5557" % SERVER).get(1000):
            print("Subscribe connection is etablished")
        else:
            print("Failed to establish Subscribe connection")
            self.assertTrue(False)

        sub.close()
        req.close()


class Reconnection(unittest.TestCase):

    def stateChange(self, req, sub, state):
        print(f"obj: {req} state change: {state}")
        if state == motorcortex.ConnectionState.CONNECTION_OK:
            print("connected")
            res = req.login("root", "vectioneer").get()
            sub.resubscribe()
            print(res)
        if state == motorcortex.ConnectionState.CONNECTION_LOST:
            print("connection lost")
        if state == motorcortex.ConnectionState.DISCONNECTED:
            print(f"disconnected")

    def notify(self, x):
        print(x)

    def test(self):
        # Open subscribe connection
        # Creating empty object for parameter tree
        parameter_tree = motorcortex.ParameterTree()

        # Loading protobuf types and hashes
        motorcortex_types = motorcortex.MessageTypes()

        # Open request connection
        req, sub = motorcortex.connect("wss://127.0.0.1:5568:5567", motorcortex_types, parameter_tree,
                                       certificate="mcx.cert.pem", timeout_ms=5000,
                                       login="root", password="vectioneer")

        sub.subscribe(["root/Comm_task/actual_cycle_max"], "test", 2000).notify(lambda x: print(f"1:{x}"))
        sub.subscribe(["root/Comm_task/actual_cycle_max"], "test1", 2000).notify(lambda x: print(f"2:{x}"))
        time.sleep(10000)
        sub.close()
        req.close()


class Login(unittest.TestCase):

    def test(self):
        # Open request connection
        motorcortex_types = motorcortex.MessageTypes()
        motorcortex_msg, motionsl_msg = motorcortex_types.load(
            [{'proto': './motorcortex-msg/motorcortex_pb2.py', 'hash': './motorcortex-msg/motorcortex_hash.json'},
             {'proto': './motorcortex-msg/motionSL_pb2.py', 'hash': './motorcortex-msg/motionSL_hash.json'}])

        parameter_tree = motorcortex.ParameterTree()
        req = motorcortex.Request(motorcortex_types, parameter_tree)
        if req.connect("ws://%s:5558" % SERVER).get(1000):
            print("Request connection is etablished")
        else:
            print("Failed to establish Request connection")
            self.assertTrue(False)

        reply = req.login("operator", "operat")
        result = reply.get()
        if result.status == motorcortex_msg.OK:
            print("Login successfull")
        else:
            print("Failed to login")
            self.assertTrue(False)

        req.close()


class DecodeEncode(unittest.TestCase):

    def test(self):
        motorcortex_types = motorcortex.MessageTypes()
        motorcortex_msg = motorcortex_types.motorcortex()
        motionsl_msg, = motorcortex_types.load(
            [{'proto': './motorcortex-msg/motionSL_pb2.py', 'hash': './motorcortex-msg/motionSL_hash.json'}])

        # check if namespaces exist
        self.assertTrue(motorcortex_types.getNamespace("motorcortex"))
        self.assertTrue(motorcortex_types.getNamespace("motion_spec"))

        # check if enums are loaded correctly
        self.assertTrue(hasattr(motorcortex_msg, 'OK'))
        # check if types are loaded
        self.assertTrue(hasattr(motorcortex_msg, 'ParameterMsg'))

        login_msg = motorcortex_types.createType('motorcortex.LoginMsg')
        login_msg.login = 'operator'
        login_msg.password = 'operat'
        res = login_msg.SerializeToString()

        login_msg1 = motorcortex_types.createType('motorcortex.LoginMsg')
        login_msg1.ParseFromString(res)

        self.assertTrue(login_msg.login == login_msg1.login)
        self.assertTrue(login_msg.password == login_msg1.password)

        # param_tree_msg = motorcortex_types.createType('motorcortex.ParameterTreeMsg')
        # param_tree_msg.hash = 01245
        # param_tree_msg.status = 1
        # res = param_tree_msg.SerializeToString()
        #
        # param_tree_msg1 = motorcortex_types.createType('motorcortex.ParameterTreeMsg')
        # param_tree_msg1.ParseFromString(res)
        #
        # self.assertTrue(param_tree_msg.hash == param_tree_msg1.hash)
        # self.assertTrue(param_tree_msg.status == param_tree_msg1.status)


class ParameterTree(unittest.TestCase):

    def test(self):
        # Open request connection
        motorcortex_types = motorcortex.MessageTypes()
        motorcortex_msg, motionsl_msg = motorcortex_types.load(
            [{'proto': './motorcortex-msg/motorcortex_pb2.py', 'hash': './motorcortex-msg/motorcortex_hash.json'},
             {'proto': './motorcortex-msg/motionSL_pb2.py', 'hash': './motorcortex-msg/motionSL_hash.json'}])

        parameter_tree = motorcortex.ParameterTree()
        req = motorcortex.Request(motorcortex_types, parameter_tree)
        if req.connect("ws://%s:5558" % SERVER).get(1000):
            print("Request connection is etablished")
        else:
            print("Failed to establish Request connection")
            self.assertTrue(False)

        reply = req.login("operator", "operat")
        result = reply.get()
        if result.status == motorcortex_msg.OK:
            print("Login successfull")
        else:
            print("Failed to login")
            self.assertTrue(False)

        # Requesting a parameter tree
        reply = req.getParameterTree()
        result = reply.get()
        parameter_tree.load(result)
        if result.status == motorcortex_msg.OK:
            print("Got parameter tree")
        else:
            print("Failed to get parameter tree")
            self.assertTrue(False)

        req.close()

    def test_hash(self):

        motorcortex.logger.setLevel(logging.DEBUG)
        motorcortex_types = motorcortex.MessageTypes()
        parameter_tree = motorcortex.ParameterTree()

        # logging.basicConfig(level=logging.INFO)
        # Open request connection
        req, sub = motorcortex.connect("wss://localhost:5568:5567", motorcortex_types, parameter_tree,
                                       certificate="mcx.cert.pem", timeout_ms=1000, login="", password="")

        path = tempfile.gettempdir() + '/' + 'mcx-python-ParameterTree-hash'
        tree = req.getParameterTree().get()
        req.saveParameterTreeFile(path, tree)
        loaded_tree = req.loadParameterTreeFile(path, motorcortex_types)

        assert (tree == loaded_tree)


class SetGetParameter(unittest.TestCase):

    def test(self):
        # Open request connection
        motorcortex_types = motorcortex.MessageTypes()
        motorcortex_msg, motionsl_msg = motorcortex_types.load(
            [{'proto': './motorcortex-msg/motorcortex_pb2.py', 'hash': './motorcortex-msg/motorcortex_hash.json'},
             {'proto': './motorcortex-msg/motionSL_pb2.py', 'hash': './motorcortex-msg/motionSL_hash.json'}])

        parameter_tree = motorcortex.ParameterTree()
        req = motorcortex.Request(motorcortex_types, parameter_tree)
        if req.connect("ws://%s:5558" % SERVER).get(1000):
            print("Request connection is etablished")
        else:
            print("Failed to establish Request connection")
            self.assertTrue(False)

        reply = req.login("operator", "operat")
        result = reply.get()
        if result.status == motorcortex_msg.OK:
            print("Login successfull")
        else:
            print("Failed to login")
            self.assertTrue(False)

        # Requesting a parameter tree
        reply = req.getParameterTree()
        result = reply.get()
        parameter_tree.load(result)
        if result.status == motorcortex_msg.OK:
            print("Got parameter tree")
        else:
            print("Failed to get parameter tree")
            self.assertTrue(False)

        reply = req.getParameter('asdasd')
        msg = reply.get()
        self.assertTrue(msg.value is None)

        reply = req.getParameter('root/MyModule1/input1')
        msg = reply.get()
        value = msg.value[0]

        value = not value
        reply = req.setParameter('root/MyModule1/input1', value)
        reply.get()

        reply = req.getParameter('root/MyModule1/input1')
        msg = reply.get()
        new_value = msg.value[0]

        self.assertTrue(value == new_value)

        req.close()


class OverwriteReleaseParameter(unittest.TestCase):

    def test(self):
        # Creating empty object for parameter tree
        parameter_tree = motorcortex.ParameterTree()

        # Loading protobuf types and hashes
        motorcortex_types = motorcortex.MessageTypes()

        # Open request connection
        req, sub = motorcortex.connect("wss://127.0.0.1:5568:5567", motorcortex_types, parameter_tree,
                                       certificate="mcx.cert.pem", timeout_ms=5000,
                                       login="root", password="vectioneer")

        ## test non-existing path
        # reply = req.overwriteParameter('asdasd', True)
        # msg = reply.get()
        # self.assertTrue(msg.value is None)

        # reply = req.releaseParameter('asdasd')
        # msg = reply.get()
        # self.assertTrue(msg.value is None)

        # test with overwrite on
        test_path = 'root/Control/dummyDouble'
        value0 = 0.0

        reply = req.setParameter(test_path, value0)
        reply.get()

        value1 = 6.0
        reply = req.overwriteParameter(test_path, value1, True)
        reply.get()

        reply = req.getParameter(test_path)
        msg = reply.get()
        new_value = msg.value[0]
        self.assertTrue(new_value == value1)  # new_value == value1

        value2 = 5.0
        reply = req.setParameter(test_path, value2)
        reply.get()

        reply = req.getParameter(test_path)
        msg = reply.get()
        new_value = msg.value[0]

        self.assertTrue(new_value != value2)  # new_value != value2
        self.assertTrue(new_value == value1)  # new_value == value1

        # test after releasing overwrite
        reply = req.releaseParameter(test_path)
        reply.get()

        reply = req.setParameter(test_path, value2)
        reply.get()

        reply = req.getParameter(test_path)
        msg = reply.get()
        new_value = msg.value[0]

        print("new_value = {}".format(new_value))
        self.assertTrue(new_value == value2)  # new_value == value2

        sub.close()
        req.close()


class StreamTest(unittest.TestCase):

    def cameraVal(self, val, id):
        print(id, val[0].timestamp, len(val[0].value))

    def test(self):
        # Creating empty object for parameter tree
        parameter_tree = motorcortex.ParameterTree()

        # Loading protobuf types and hashes
        motorcortex_types = motorcortex.MessageTypes()

        # Open request connection
        req, sub = motorcortex.connect("ws://localhost:5558:5557", motorcortex_types, parameter_tree,
                                       certificate="mcx.cert.pem", timeout_ms=5000,
                                       login="", password="")

        subscription = sub.subscribe(["root/Comm_task/utilization_max",
                                      "root/Control/Camera/fameCounter"], "test", 1)
        subscription.get()
        subscription1 = sub.subscribe("root/test/varD", "camera", 1)
        subscription1.get()
        subscription.notify(lambda a: newVal(a, 0))
        subscription1.notify(lambda a: self.cameraVal(a, 1))
        time.sleep(500)

        req.close()
        sub.close()


class ConnectionTimeout(unittest.TestCase):
    def stateChange(self, req, state):
        print(f"obj: {req} state change: {state}")
        if state == motorcortex.ConnectionState.CONNECTION_OK:
            print("connected")
            res = req.login("root", "vectioneer").get()
            print(res)
        if state == motorcortex.ConnectionState.CONNECTION_LOST:
            print("connection lost")
        if state == motorcortex.ConnectionState.DISCONNECTED:
            print(f"disconnected")

    def done(self, arg1):
        print(f"Connection done: {arg1}")

    def failed(self):
        print("Connection failed")

    def test(self):
        # Creating empty object for parameter tree
        parameter_tree = motorcortex.ParameterTree()

        # Loading protobuf types and hashes
        motorcortex_types = motorcortex.MessageTypes()

        # Open request connection
        req, sub = motorcortex.connect("wss://localhost:5568:5567", motorcortex_types, parameter_tree,
                                       certificate="mcx.cert.pem", timeout_ms=5000,
                                       login="test", password="test", state_update=self.stateChange)

        while True:
            res = req.getParameter("root/Control/testB/input2")
            val = res.get()
            if val.status == motorcortex.OK:
                print(val.value)
            time.sleep(1)

        req.close()


class GetParameter(unittest.TestCase):
    def test(self):
        # Creating empty object for parameter tree
        parameter_tree = motorcortex.ParameterTree()

        # Loading protobuf types and hashes
        motorcortex_types = motorcortex.MessageTypes()

        # Open request connection
        req, sub = motorcortex.connect("wss://192.168.179.85:5568:5567", motorcortex_types, parameter_tree,
                                       certificate="mcx.cert.pem", timeout_ms=5000,
                                       login="test", password="test")

        rep = req.getParameterList(["root/Logic/mode",
                                    "root/Logic/modeCommand"]).get()
        print(f"{rep}")

        rep.close()


class SubParameter(unittest.TestCase):
    counter = 0

    def newValue(self, value):
        self.counter = self.counter + 1

    def test(self):
        # Open request connection
        motorcortex_types = motorcortex.MessageTypes()
        motorcortex_msg, motionsl_msg = motorcortex_types.load(
            [{'proto': './motorcortex-msg/motorcortex_pb2.py', 'hash': './motorcortex-msg/motorcortex_hash.json'},
             {'proto': './motorcortex-msg/motionSL_pb2.py', 'hash': './motorcortex-msg/motionSL_hash.json'}])

        parameter_tree = motorcortex.ParameterTree()
        req = motorcortex.Request(motorcortex_types, parameter_tree)
        if req.connect("wss://%s:5568" % SERVER, certificate="mcx.cert.pem").get(5000):
            print("Request connection is etablished")
        else:
            print("Failed to establish Request connection")
            self.assertTrue(False)

        reply = req.login("operator", "operat")
        result = reply.get()
        if result.status == motorcortex_msg.OK:
            print("Login successfull")
        else:
            print("Failed to login")
            self.assertTrue(False)

        # Requesting a parameter tree
        reply = req.getParameterTree()
        result = reply.get()
        parameter_tree.load(result)
        if result.status == motorcortex_msg.OK:
            print("Got parameter tree")
        else:
            print("Failed to get parameter tree")
            self.assertTrue(False)

        sub = motorcortex.Subscribe(req, motorcortex_types)
        if sub.connect("wss://%s:5567" % SERVER, certificate="mcx.cert.pem").get(1000):
            print("Subscribe connection is etablished")
        else:
            print("Failed to establish Subscribe connection")
            self.assertTrue(False)

        subscription = sub.subscribe(['fsdf'], 'group1', 100)
        self.assertTrue(subscription.get() is None)

        subscription = sub.subscribe(['root/Control/dummyDouble'], 'group1', 100)
        self.assertTrue(subscription.get())

        subscription.notify(self.newValue)
        timestamp = 0
        for x in range(400):
            print(f"timestamp: {subscription.read()[0].timestamp}, value: {subscription.read()[0].value}")
            time.sleep(1)

        self.assertTrue(sub.unsubscribe(subscription).get().status == motorcortex.OK)

        self.assertTrue(self.counter > 0)
        self.assertTrue(timestamp.sec > 0)

        sub.close()
        req.close()


class SetString(unittest.TestCase):
    def test(self):
        parameter_tree = motorcortex.ParameterTree()
        # Open request and subscribe connection
        req, sub = motorcortex.connect("wss://localhost:5568:5567", motorcortex.MessageTypes(), parameter_tree,
                                       certificate="mcx.cert.pem", timeout_ms=1000,
                                       login="root", password="secret")
        res = req.setParameter("root/Control/dummyChar", "Hallo robot!")
        res.get()


class SetParameterWithOffset(unittest.TestCase):
    def test(self):
        parameter_tree = motorcortex.ParameterTree()
        # Open request and subscribe connection
        req, sub = motorcortex.connect("wss://localhost:5568:5567", motorcortex.MessageTypes(), parameter_tree,
                                       certificate="mcx.cert.pem", timeout_ms=1000,
                                       login="root", password="secret")
        res = req.setParameter("root/Control/hostInJointTrajectory", [6, 5])
        res.get()


if __name__ == '__main__':
    unittest.main()
